unit LapTimeChart;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, ExtCtrls, Graphics, Forms;

type
  TLapTimes = class;
  TLapTimeChart = class;
  TLapTimeItemClass = class of TLapTimeItem;

  TLTCreateItemClassEvent = procedure(Sender: TLapTimeChart; var ItemClass: TLapTimeItemClass) of object;

  TTimeData = array of Single;

  TLTMouseEnterEvent = procedure(Sender: TObject; Index, LapCount: Integer) of object;

  TLapTimeItem = class(TCollectionItem)
  private
    FID: Integer;
    FColor: TColor;
    FData: TTimeData;
    FWidth: Integer;
    procedure SetColor(const Value: TColor);
    procedure SetID(const Value: Integer);
    function GetData(Index: Integer): Single;
    procedure SetData(Index: Integer; const Value: Single);
    function GetCount: Integer;
    procedure SetCount(const Value: Integer);
    procedure SetWidth(const Value: Integer);
  protected
    function GetDisplayName: string; override;
    procedure AddData(Value: Single);
    procedure ClearData;
    procedure DeleteData(Index: Integer);
    property Count: Integer read GetCount write SetCount;
  public
    constructor Create(Collection: TCollection); override;
    procedure Assign(Source: TPersistent); override;
    class procedure Error(const Msg: string; Data: Integer); overload; virtual;
    class procedure Error(Msg: PResStringRec; Data: Integer); overload;
    property Data[Index: Integer]: Single read GetData write SetData;
  published
    property ID: Integer read FID write SetID default 0;
    property Color: TColor read FColor write SetColor default clBlue;
    property Width: Integer read FWidth write SetWidth default 1;
  end;

  TLapTimes = class(TCollection)
  private
    FChart: TLapTimeChart;
    function GetItem(Index: Integer): TLapTimeItem;
    procedure SetItem(Index: Integer; Value: TLapTimeItem);
  protected
    function GetOwner: TPersistent; override;
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(Chart: TLapTimeChart);
    function Add: TLapTimeItem;
    function AddItem(Item: TLapTimeItem; Index: Integer): TLapTimeItem;
    function Insert(Index: Integer): TLapTimeItem;
    property Items[Index: Integer]: TLapTimeItem read GetItem write SetItem; default;
  end;

  TLapTimeChart = class(TCustomPanel)
  private
    { Private 錾 }
    FLineColor: TColor;
    FTimeColor: TColor;
    FLapColor: TColor;
    FTargetColor: TColor;
    FMinTime: Byte;
    FMaxTime: Byte;
    FCount: Integer;
    FTarget: Single;
    FTimes: TLapTimes;
    FOnMouseEnter: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
    FOnCreateItemClass: TLTCreateItemClassEvent;
    FOnLTMouseEnterEvent: TLTMouseEnterEvent;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMHintShow(var Message: TMessage); message CM_HINTSHOW;
    procedure SetLapColor(const Value: TColor);
    procedure SetLineColor(const Value: TColor);
    procedure SetTimeColor(const Value: TColor);
    procedure SetMaxTime(const Value: Byte);
    procedure SetMinTime(const Value: Byte);
    procedure SetCount(const Value: Integer);
    procedure SetTimes(const Value: TLapTimes);
    procedure SetTarget(const Value: Single);
    procedure SetTargetColor(const Value: TColor);
    procedure SetItemCount(Item: TLapTimeItem);
    function IsShowHint(out AHint: string; out ARect: TRect): Boolean; overload;
    function IsShowHint(out Index, LapCount: Integer): Boolean; overload;
    procedure DoLTMouseEnter(Index, LapCount: Integer);
  protected
    { Protected 錾 }
    function CanAutoSize(var NewWidth, NewHeight: Integer): Boolean; override;
    procedure Paint; override;
    function CreateItem: TLapTimeItem;
    function CreateItems: TLapTimes;
    function GetItemClass: TLapTimeItemClass;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
  public
    { Public 錾 }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property DockManager;
  published
    { Published 錾 }
    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property BiDiMode;
    property BorderWidth;
    property Caption;
    property Color default clWhite;
    property Count: Integer read FCount write SetCount default 30;
    property LineColor: TColor read FLineColor write SetLineColor default clBlack;
    property TimeColor: TColor read FTimeColor write SetTimeColor default clBlack;
    property LapColor: TColor read FLapColor write SetLapColor default clBlack;
    property Constraints;
    property Ctl3D;
    property UseDockManager default True;
    property DockSite;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FullRepaint;
    property Font;
    property Locked;
    property MinTime: Byte read FMinTime write SetMinTime default 10;
    property MaxTime: Byte read FMaxTime write SetMaxTime default 25;
    property ParentBiDiMode;
    property ParentBackground;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint default False;
    property PopupMenu;
    property ShowHint default True;
    property TabOrder;
    property TabStop;
    property TargetColor: TColor read FTargetColor write SetTargetColor default clRed;
    property Target: Single read FTarget Write SetTarget;
    property Times: TLapTimes read FTimes write SetTimes;
    property Visible;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDockDrop;
    property OnDockOver;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseDown;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnLTMouseEnter: TLTMouseEnterEvent read FOnLTMouseEnterEvent write FOnLTMouseEnterEvent;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
    property OnUnDock;
    property OnCreateItemClass: TLTCreateItemClassEvent read FOnCreateItemClass write FOnCreateItemClass;
  end;

procedure Register;

implementation

uses
  RTLConsts;

type
  TLine = record
    S: Integer;
    E: Integer;
  end;
  
var
  XLeng, YLeng, XSpan, YSpan: Integer;
  XLine, YLine: TLine;

procedure Register;
begin
  RegisterComponents('Custom', [TLapTimeChart]);
end;

{ TLapTimeItem }

procedure TLapTimeItem.AddData(Value: Single);
var
  C: Integer;
begin
  C := Length(FData);
  SetLength(FData, C + 1);
  FData[C] := Value;
  Changed(False);  
end;

procedure TLapTimeItem.Assign(Source: TPersistent);
begin
  if Source is TLapTimeItem then
  begin
(*
    Text := TStatusPanel(Source).Text;
    Width := TStatusPanel(Source).Width;
    Alignment := TStatusPanel(Source).Alignment;
    Bevel := TStatusPanel(Source).Bevel;
    Style := TStatusPanel(Source).Style;
*)
  end
  else
    inherited Assign(Source);
end;

procedure TLapTimeItem.ClearData;
begin
  SetLength(FData, 0);
  Changed(False);
end;

constructor TLapTimeItem.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FID := 0;
  FColor  := clBlue;
  SetLength(FData, 0);
end;

class procedure TLapTimeItem.Error(const Msg: string; Data: Integer);

  function ReturnAddr: Pointer;
  asm
          MOV     EAX,[EBP+4]
  end;

begin
  raise EListError.CreateFmt(Msg, [Data]) at ReturnAddr;
end;

procedure TLapTimeItem.DeleteData(Index: Integer);
var
  i: Integer;
begin
  if (Index < 0) or (Index >= Count) then Error(@SListIndexError, Index);

  for i := Index to Count - 1 do
    if Index < Count - 1 then FData[Index] := FData[Index + 1];
  SetLength(FData, Length(FData) - 1);
  Changed(False);
end;

class procedure TLapTimeItem.Error(Msg: PResStringRec; Data: Integer);
begin
  TList.Error(LoadResString(Msg), Data);
end;

function TLapTimeItem.GetCount: Integer;
begin
  Result  := Length(FData);
end;

function TLapTimeItem.GetData(Index: Integer): Single;
begin
  if (Index < 0) or (Index >= Count) then Error(@SListIndexError, Index);
  Result := FData[Index];
end;

function TLapTimeItem.GetDisplayName: string;
begin
  Result := '';//Text;
  if Result = '' then Result := inherited GetDisplayName;
end;

procedure TLapTimeItem.SetColor(const Value: TColor);
begin
  if FColor <> Value then
  begin
    FColor := Value;
    Changed(False);
  end;
end;

procedure TLapTimeItem.SetData(Index: Integer; const Value: Single);
begin
  if (Index < 0) or (Index >= Count) then Error(@SListIndexError, Index);
  if FData[Index] <> Value then
  begin
    FData[Index] := Value;
    Changed(False);
  end;
end;

procedure TLapTimeItem.SetID(const Value: Integer);
begin
  if FID <> Value then
  begin
    FID := Value;
    Changed(False);
  end;
end;

procedure TLapTimeItem.SetCount(const Value: Integer);
begin
  SetLength(FData, Value);
end;

procedure TLapTimeItem.SetWidth(const Value: Integer);
begin
  if (FWidth <> Value) and (Value > 0) then
  begin
    FWidth := Value;
    Changed(False);
  end;
end;

{ TLapTimes }

function TLapTimes.Add: TLapTimeItem;
var
  i: Integer;
begin
  Result := TLapTimeItem(inherited Add);
  if Assigned(FChart) then
  begin
    Result.SetCount(FChart.Count);
    for i := 0 to FChart.Count - 1 do Result.Data[i] := 0;
  end;
end;

function TLapTimes.AddItem(Item: TLapTimeItem;
  Index: Integer): TLapTimeItem;
begin
  if Item = nil then
    Result := FChart.CreateItem
  else
    Result := Item;
  if Assigned(Result) then
  begin
    Result.Collection := Self;
    if Index < 0 then
      Index := Count - 1;
    Result.Index := Index;
    if Assigned(FChart) then FChart.SetItemCount(Result);    
  end;
end;

constructor TLapTimes.Create(Chart: TLapTimeChart);
begin
  if Chart <> nil then
    inherited Create(Chart.GetItemClass)
  else
    inherited Create(TLapTimeItem);
  FChart := Chart;
end;

function TLapTimes.GetItem(Index: Integer): TLapTimeItem;
begin
  Result := TLapTimeItem(inherited GetItem(Index));
end;

function TLapTimes.GetOwner: TPersistent;
begin
  Result := FChart;
end;

function TLapTimes.Insert(Index: Integer): TLapTimeItem;
begin
  Result := AddItem(nil, Index);
end;

procedure TLapTimes.SetItem(Index: Integer; Value: TLapTimeItem);
begin
  inherited SetItem(Index, Value);
end;

procedure TLapTimes.Update(Item: TCollectionItem);
begin
  FChart.Invalidate;
end;

{ TLapTimeChart }

function TLapTimeChart.CanAutoSize(var NewWidth,
  NewHeight: Integer): Boolean;
begin
  Result  := True;
end;

procedure TLapTimeChart.CMFontChanged(var Message: TMessage);
begin
  inherited;
end;

procedure TLapTimeChart.CMHintShow(var Message: TMessage);
var
  OutHint: string;
  OutRect: TRect;
begin
  if IsShowHint(OutHint, OutRect) then
  begin
    with TCMHintShow(Message).HintInfo^ do
    begin
      CursorRect := OutRect;
      HintStr := OutHint;
      HintPos.X := CursorRect.Left + 16;
      HintPos.Y := CursorRect.Bottom + 4;
      HintPos := ClientToScreen(HintPos);
    end;
    Message.Result := 0;
  end else
    inherited;
end;

procedure TLapTimeChart.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if Assigned(FOnMouseEnter) then FOnMouseEnter(self);
end;

procedure TLapTimeChart.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if Assigned(FOnMouseLeave) then FOnMouseLeave(self);
end;

procedure TLapTimeChart.CMTextChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

constructor TLapTimeChart.Create(AOwner: TComponent);
begin
  FMinTime  := 10;
  FMaxTime  := 25;
  FCount    := 30;
  FTarget   := 0;
  inherited;

  FTimes  := CreateItems;

  ParentColor       := False;
  Color             := clWhite;
  ParentBackground  := False;
  ParentShowHint    := False;
  ShowHint          := True;

  BorderWidth := 6;
  BorderStyle := bsNone;

  BevelInner  := bvNone;
  BevelOuter  := bvNone;
  BevelWidth  := 1;

  FLineColor    := clBlack;
  FTimeColor    := clBlack;
  FLapColor     := clBlack;
  FTargetColor  := clRed;
end;

function TLapTimeChart.CreateItem: TLapTimeItem;
var
  LClass: TLapTimeItemClass;
begin
  LClass := GetItemClass;
  if Assigned(FOnCreateItemClass) then
    FOnCreateItemClass(Self, LClass);
  Result := LClass.Create(Times);
end;

function TLapTimeChart.CreateItems: TLapTimes;
begin
  Result := TLapTimes.Create(Self);
end;

destructor TLapTimeChart.Destroy;
begin
  FTimes.Free;
  inherited Destroy;
end;

procedure TLapTimeChart.DoLTMouseEnter(Index, LapCount: Integer);
begin
  if Assigned(FOnLTMouseEnterEvent) then
    FOnLTMouseEnterEvent(Self, Index, LapCount);
end;

function TLapTimeChart.GetItemClass: TLapTimeItemClass;
begin
  Result := TLapTimeItem;
end;

function TLapTimeChart.IsShowHint(out AHint: string; out ARect: TRect): Boolean;
var
  W, Index, LapCount: Integer;
  C: TPoint;
begin
  Result  := IsShowHint(Index, LapCount);
  if Result then
  begin
    C := Point(XLine.S + XSpan * (LapCount + 1), YLeng - Trunc(YSpan * (Times[Index].Data[LapCount] - MinTime)) + BorderWidth);
    W := Trunc(Times[Index].Width / 2);
    ARect.Left   := C.X - (2 + W);
    ARect.Right  := C.X + (3 + W);
    ARect.Top    := C.Y - (2 + W);
    ARect.Bottom := C.Y + (3 + W);
    AHint   := Format('ID:%d, Lap:%d, Time:%.2f', [Times[Index].ID, LapCount + 1, Times[Index].Data[LapCount]]);
  end;
end;

function TLapTimeChart.IsShowHint(out Index, LapCount: Integer): Boolean;
var
  P, C: TPoint;
  W, i, n: Integer;
  OutRect: TRect;
begin
  Result  := False;
  if Times.Count = 0 then Exit;

  if GetCursorPos(P) then
  begin
    P := ScreenToClient(P);
    for i := 0 to Times.Count - 1 do
    begin
      W := Trunc(Times[i].Width / 2);
      for n := 0 to Times[i].Count - 1 do
      begin
        C := Point(XLine.S + XSpan * (n + 1), YLeng - Trunc(YSpan * (Times[i].Data[n] - MinTime)) + BorderWidth);
        OutRect.Left   := C.X - (2 + W);
        OutRect.Right  := C.X + (3 + W);
        OutRect.Top    := C.Y - (2 + W);
        OutRect.Bottom := C.Y + (3 + W);
        if PtInRect(OutRect, P) then
        begin
          Result  := True;
          Index   := i;
          LapCount:= n;
          Exit;
        end;
      end;
    end;
  end;
end;

procedure TLapTimeChart.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  Index, LapCount: Integer;
begin
  inherited;
  if IsShowHint(Index, LapCount) then DoLTMouseEnter(Index, LapCount);
end;

procedure TLapTimeChart.Paint;
var
  Margin,
  i, n, P: Integer;
  TimeFont: TFont;
  LogFont: TLogFont;

  procedure DrawCircle(W: Integer; Center: TPoint);
  var
    TimeRect: TRect;
  begin
    W := Trunc(W / 2);
    TimeRect.Left   := Center.X - (2 + W);
    TimeRect.Right  := Center.X + (3 + W);
    TimeRect.Top    := Center.Y - (2 + W);
    TimeRect.Bottom := Center.Y + (3 + W);
    Canvas.Ellipse(TimeRect);
  end;

  procedure DrawTimeStr(C: Integer);
  begin
    Canvas.TextOut(
      BorderWidth + Canvas.TextHeight('Wg') + 5 + (Canvas.TextWidth('00') - Canvas.TextWidth(Format('%d', [(MaxTime - C)]))),
      P - Trunc(Canvas.TextHeight('Wg') / 2),
      Format('%d', [(MaxTime - C)])      
    );
  end;
  
  procedure DrawLapStr(C: Integer);
  begin
    P := XLine.S + XSpan * C;
    Canvas.TextOut(
      P - Trunc(Canvas.TextWidth(Format('%d', [C])) / 2),
      YLine.E + 5,
      Format('%d', [C])
    );
  end;
begin
//  inherited;
  Canvas.Font.Assign(Self.Font);
  Canvas.Pen.Style    := psClear;
  Canvas.Brush.Style  := bsSolid;
  Canvas.Brush.Color  := Color;
  Canvas.FillRect(ClientRect);

  XLine.S  := BorderWidth + Canvas.TextHeight('Wg') + 5 + Canvas.TextWidth('00') + 5;
  XLine.E  := ClientWidth - BorderWidth;
  YLine.S  := BorderWidth;
  YLine.E  := ClientHeight - (BorderWidth + Canvas.TextHeight('Wg') + 5 + Canvas.TextHeight('Wg') + 5);

  Canvas.Pen.Style  := psSolid;
  Canvas.Pen.Width  := 1;
  Canvas.Pen.Color  := LineColor;

  // Y C
  Canvas.Polyline([Point(XLine.S, YLine.S), Point(XLine.S, YLine.E)]);

  // X C
  Canvas.Polyline([Point(XLine.S, YLine.E), Point(XLine.E, YLine.E)]);

  // ^Cx
  Canvas.Font.Color := TimeColor;
  YLeng := YLine.E - YLine.S;
  // tHg]
  TimeFont  := TFont.Create;
  try
    TimeFont.Assign(Canvas.Font);
    GetObject(TimeFont.Handle, SizeOf(LogFont), @LogFont);
    LogFont.lfEscapement  := 90 * 10;
    LogFont.lfOrientation := 90 * 10;
    TimeFont.Handle := CreateFontIndirect(LogFont);
    Canvas.Font.Assign(TimeFont);
  finally
    TimeFont.Free;
  end;
  Canvas.TextOut(
    BorderWidth,
    Trunc(YLeng / 2) + Canvas.TextWidth('bv^C(b)'),
    'bv^C(b)'
  );

  // tHgɖ߂
  Canvas.Font.Assign(Self.Font);

  // bv()
  Canvas.Font.Color := LapColor;
  XLeng := XLine.E - XLine.S;
  // 񐔃x
  Canvas.TextOut(
    XLine.S + Trunc((XLeng - Canvas.TextWidth('bv()')) / 2),
    ClientHeight - BorderWidth - Canvas.TextHeight('Wg'),
    'bv()'
  );

  // bv^CCƕ
  if MaxTime - MinTime = 0 then Exit;
  Canvas.Font.Color := TimeColor;
  Margin  := YLeng mod Trunc(MaxTime - MinTime);
  YSpan    := Trunc(YLeng / Trunc(MaxTime - MinTime));
  for i := 0 to Trunc(MaxTime - MinTime) do
  begin
    P := YLine.S + Margin + YSpan * i;
    Canvas.Polyline([
      Point(XLine.S, P),
      Point(XLine.E, P)
    ]);
    if YSpan * 2 <= Canvas.TextHeight('Wg') then
    begin
      if i mod 5 = 0 then DrawTimeStr(i);
    end
    else if YSpan <= Canvas.TextHeight('Wg') then
    begin
      if Odd(i) then DrawTimeStr(i); 
    end else
      DrawTimeStr(i);
  end;

  // 񐔕
  Canvas.Font.Color := LapColor;
  if FCount = 0 then Exit;
  XSpan    := Trunc(XLeng / Count);

  for i := 1 to Count do
  begin
    if XSpan * 2 < Canvas.TextWidth('000') then
    begin
      if i Mod 5 = 0 then DrawLapStr(i); 
    end
    else if XSpan < Canvas.TextWidth('000') then
    begin
      if not Odd(i) then DrawLapStr(i);
    end else
      DrawLapStr(i);
  end;

  if Trunc(Target) > 0 then
  begin
    // ^[QbgC`
    Canvas.Pen.Color  := TargetColor;
    Canvas.Pen.Width  := 1;
    Canvas.Pen.Style  := psSolid;
    Canvas.Polyline([
      Point(XLine.S, YLeng - Trunc(YSpan * (Target - MinTime)) + BorderWidth),
      Point(XLine.E, YLeng - Trunc(YSpan * (Target - MinTime)) + BorderWidth)
    ]);
  end;

  // bv^C`
  for i := 0 to Times.Count - 1 do
  begin
    Canvas.Pen.Color    := Times[i].Color;
    Canvas.Pen.Style    := psSolid;
    Canvas.Brush.Color  := Times[i].Color;
    Canvas.Brush.Style  := bsSolid;
    for n := 0 to Times[i].Count - 1 do
      if Trunc(Times[i].Data[n]) > 0 then
      begin
        Canvas.Pen.Width  := 1;
        DrawCircle(Times[i].Width, Point(XLine.S + XSpan * (n + 1), YLeng - Trunc(YSpan * (Times[i].Data[n] - MinTime)) + BorderWidth));
        if n = 0 then Continue;
        Canvas.Pen.Width  := Times[i].Width;
        Canvas.Polyline([
          Point(XLine.S + XSpan * (n), YLeng - Trunc(YSpan * (Times[i].Data[n - 1] - MinTime)) + BorderWidth),
          Point(XLine.S + XSpan * (n + 1), YLeng - Trunc(YSpan * (Times[i].Data[n] - MinTime)) + BorderWidth)
        ]);
      end;
  end;
end;

procedure TLapTimeChart.SetCount(const Value: Integer);
var
  i: Integer;
begin
  if FCount <> Value then
  begin
    FCount := Value;
    for i := 0 to Times.Count - 1 do SetItemCount(Times[i]);
    Invalidate;
  end;
end;

procedure TLapTimeChart.SetItemCount(Item: TLapTimeItem);
var
  i, C: Integer;
begin
  C := Count - Item.Count;
  if Count > 0 then
    // 
    for i := 0 to C - 1 do Item.AddData(0)
  else
    // 
    Item.SetCount(C);
end;

procedure TLapTimeChart.SetLapColor(const Value: TColor);
begin
  if FLapColor <> Value then
  begin
    FLapColor := Value;
    Invalidate;
  end;
end;

procedure TLapTimeChart.SetLineColor(const Value: TColor);
begin
  if FLineColor <> Value then
  begin
    FLineColor := Value;
    Invalidate;
  end;
end;

procedure TLapTimeChart.SetMaxTime(const Value: Byte);
begin
  if FMaxTime <> Value then
  begin
    FMaxTime := Value;
    Invalidate;
  end;
end;

procedure TLapTimeChart.SetMinTime(const Value: Byte);
begin
  if FMinTime <> Value then
  begin
    FMinTime := Value;
    Invalidate;
  end;
end;

procedure TLapTimeChart.SetTarget(const Value: Single);
begin
  if FTarget <> Value then
  begin
    FTarget := Value;
    Invalidate;
  end;
end;

procedure TLapTimeChart.SetTargetColor(const Value: TColor);
begin
  if FTargetColor <> Value then
  begin
    FTargetColor := Value;
    Invalidate;
  end;
end;

procedure TLapTimeChart.SetTimeColor(const Value: TColor);
begin
  if FTimeColor <> Value then
  begin
    FTimeColor := Value;
    Invalidate;
  end;
end;

procedure TLapTimeChart.SetTimes(const Value: TLapTimes);
begin
  FTimes.Assign(Value);
end;

end.
